# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["BillingGroup"]


class BillingGroup(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    deleted_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was removed."""

    name: Optional[str] = None
    """A user-specified name for the billing group"""

    organization_id: Optional[str] = None
    """Identifies the organization that owns the resource."""

    record_type: Optional[Literal["billing_group"]] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .entity_type import EntityType
from .stock_exchange import StockExchange
from .alt_business_id_type import AltBusinessIDType
from .brand_identity_status import BrandIdentityStatus
from .brand_optional_attributes import BrandOptionalAttributes

__all__ = ["TelnyxBrand"]


class TelnyxBrand(BaseModel):
    """Telnyx-specific extensions to The Campaign Registry's `Brand` type"""

    brand_relationship: Literal["BASIC_ACCOUNT", "SMALL_ACCOUNT", "MEDIUM_ACCOUNT", "LARGE_ACCOUNT", "KEY_ACCOUNT"] = (
        FieldInfo(alias="brandRelationship")
    )
    """Brand relationship to the CSP."""

    country: str
    """ISO2 2 characters country code. Example: US - United States"""

    display_name: str = FieldInfo(alias="displayName")
    """Display or marketing name of the brand."""

    email: str
    """Valid email address of brand support contact."""

    entity_type: EntityType = FieldInfo(alias="entityType")
    """Entity type behind the brand. This is the form of business establishment."""

    vertical: str
    """Vertical or industry segment of the brand."""

    alt_business_id: Optional[str] = FieldInfo(alias="altBusinessId", default=None)
    """Alternate business identifier such as DUNS, LEI, or GIIN"""

    alt_business_id_type: Optional[AltBusinessIDType] = FieldInfo(alias="altBusinessIdType", default=None)
    """An enumeration."""

    brand_id: Optional[str] = FieldInfo(alias="brandId", default=None)
    """Unique identifier assigned to the brand."""

    business_contact_email: Optional[str] = FieldInfo(alias="businessContactEmail", default=None)
    """Business contact email.

    Required if `entityType` is `PUBLIC_PROFIT`.
    """

    city: Optional[str] = None
    """City name"""

    company_name: Optional[str] = FieldInfo(alias="companyName", default=None)
    """(Required for Non-profit/private/public) Legal company name."""

    created_at: Optional[str] = FieldInfo(alias="createdAt", default=None)
    """Date and time that the brand was created at."""

    csp_id: Optional[str] = FieldInfo(alias="cspId", default=None)
    """Unique identifier assigned to the csp by the registry."""

    ein: Optional[str] = None
    """(Required for Non-profit) Government assigned corporate tax ID.

    EIN is 9-digits in U.S.
    """

    failure_reasons: Optional[str] = FieldInfo(alias="failureReasons", default=None)
    """Failure reasons for brand"""

    first_name: Optional[str] = FieldInfo(alias="firstName", default=None)
    """First name of business contact."""

    identity_status: Optional[BrandIdentityStatus] = FieldInfo(alias="identityStatus", default=None)
    """The verification status of an active brand"""

    ip_address: Optional[str] = FieldInfo(alias="ipAddress", default=None)
    """IP address of the browser requesting to create brand identity."""

    is_reseller: Optional[bool] = FieldInfo(alias="isReseller", default=None)
    """Indicates whether this brand is known to be a reseller"""

    last_name: Optional[str] = FieldInfo(alias="lastName", default=None)
    """Last name of business contact."""

    mobile_phone: Optional[str] = FieldInfo(alias="mobilePhone", default=None)
    """Valid mobile phone number in e.164 international format."""

    mock: Optional[bool] = None
    """Mock brand for testing purposes"""

    optional_attributes: Optional[BrandOptionalAttributes] = FieldInfo(alias="optionalAttributes", default=None)

    phone: Optional[str] = None
    """Valid phone number in e.164 international format."""

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """Postal codes. Use 5 digit zipcode for United States"""

    reference_id: Optional[str] = FieldInfo(alias="referenceId", default=None)
    """Unique identifier Telnyx assigned to the brand - the brandId"""

    state: Optional[str] = None
    """State. Must be 2 letters code for United States."""

    status: Optional[Literal["OK", "REGISTRATION_PENDING", "REGISTRATION_FAILED"]] = None
    """Status of the brand"""

    stock_exchange: Optional[StockExchange] = FieldInfo(alias="stockExchange", default=None)
    """(Required for public company) stock exchange."""

    stock_symbol: Optional[str] = FieldInfo(alias="stockSymbol", default=None)
    """(Required for public company) stock symbol."""

    street: Optional[str] = None
    """Street number and name."""

    tcr_brand_id: Optional[str] = FieldInfo(alias="tcrBrandId", default=None)
    """Unique identifier assigned to the brand by the registry."""

    universal_ein: Optional[str] = FieldInfo(alias="universalEin", default=None)
    """Universal EIN of Brand, Read Only."""

    updated_at: Optional[str] = FieldInfo(alias="updatedAt", default=None)
    """Date and time that the brand was last updated at."""

    webhook_failover_url: Optional[str] = FieldInfo(alias="webhookFailoverURL", default=None)
    """Failover webhook to which brand status updates are sent."""

    webhook_url: Optional[str] = FieldInfo(alias="webhookURL", default=None)
    """Webhook to which brand status updates are sent."""

    website: Optional[str] = None
    """Brand website URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["OpenAIResource", "AsyncOpenAIResource"]


class OpenAIResource(SyncAPIResource):
    @cached_property
    def embeddings(self) -> EmbeddingsResource:
        """
        OpenAI-compatible embeddings endpoints for generating vector representations of text
        """
        return EmbeddingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> OpenAIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return OpenAIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OpenAIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return OpenAIResourceWithStreamingResponse(self)


class AsyncOpenAIResource(AsyncAPIResource):
    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResource:
        """
        OpenAI-compatible embeddings endpoints for generating vector representations of text
        """
        return AsyncEmbeddingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOpenAIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOpenAIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOpenAIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncOpenAIResourceWithStreamingResponse(self)


class OpenAIResourceWithRawResponse:
    def __init__(self, openai: OpenAIResource) -> None:
        self._openai = openai

    @cached_property
    def embeddings(self) -> EmbeddingsResourceWithRawResponse:
        """
        OpenAI-compatible embeddings endpoints for generating vector representations of text
        """
        return EmbeddingsResourceWithRawResponse(self._openai.embeddings)


class AsyncOpenAIResourceWithRawResponse:
    def __init__(self, openai: AsyncOpenAIResource) -> None:
        self._openai = openai

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResourceWithRawResponse:
        """
        OpenAI-compatible embeddings endpoints for generating vector representations of text
        """
        return AsyncEmbeddingsResourceWithRawResponse(self._openai.embeddings)


class OpenAIResourceWithStreamingResponse:
    def __init__(self, openai: OpenAIResource) -> None:
        self._openai = openai

    @cached_property
    def embeddings(self) -> EmbeddingsResourceWithStreamingResponse:
        """
        OpenAI-compatible embeddings endpoints for generating vector representations of text
        """
        return EmbeddingsResourceWithStreamingResponse(self._openai.embeddings)


class AsyncOpenAIResourceWithStreamingResponse:
    def __init__(self, openai: AsyncOpenAIResource) -> None:
        self._openai = openai

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResourceWithStreamingResponse:
        """
        OpenAI-compatible embeddings endpoints for generating vector representations of text
        """
        return AsyncEmbeddingsResourceWithStreamingResponse(self._openai.embeddings)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["FaxDelivered", "Data", "DataPayload", "Meta"]


class DataPayload(BaseModel):
    call_duration_secs: Optional[int] = None
    """The duration of the call in seconds."""

    client_state: Optional[str] = None
    """State received from a command."""

    connection_id: Optional[str] = None
    """The ID of the connection used to send the fax."""

    direction: Optional[Literal["inbound", "outbound"]] = None
    """The direction of the fax."""

    fax_id: Optional[str] = None
    """Identifies the fax."""

    from_: Optional[str] = FieldInfo(alias="from", default=None)
    """The phone number, in E.164 format, the fax will be sent from."""

    media_name: Optional[str] = None
    """The media_name used for the fax's media.

    Must point to a file previously uploaded to api.telnyx.com/v2/media by the same
    user/organization. media_name and media_url/contents can't be submitted
    together.
    """

    original_media_url: Optional[str] = None
    """The original URL to the PDF used for the fax's media.

    If media_name was supplied, this is omitted
    """

    page_count: Optional[int] = None
    """Number of transferred pages"""

    status: Optional[Literal["delivered"]] = None
    """The status of the fax."""

    to: Optional[str] = None
    """The phone number, in E.164 format, the fax will be sent to or SIP URI"""

    user_id: Optional[str] = None
    """Identifier of the user to whom the fax belongs"""


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["fax.delivered"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class Meta(BaseModel):
    """Metadata about the webhook delivery."""

    attempt: Optional[int] = None
    """The delivery attempt number."""

    delivered_to: Optional[str] = None
    """The URL the webhook was delivered to."""


class FaxDelivered(BaseModel):
    data: Optional[Data] = None

    meta: Optional[Meta] = None
    """Metadata about the webhook delivery."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["ListRetrieveByZoneResponse", "Data", "DataNumber"]


class DataNumber(BaseModel):
    country: Optional[str] = None

    number: Optional[str] = None


class Data(BaseModel):
    number_of_channels: Optional[int] = None

    numbers: Optional[List[DataNumber]] = None

    zone_id: Optional[str] = None

    zone_name: Optional[str] = None


class ListRetrieveByZoneResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PhoneNumberInitializeVerificationParams"]


class PhoneNumberInitializeVerificationParams(TypedDict, total=False):
    display_name: Required[str]

    phone_number: Required[str]

    language: str

    verification_method: Literal["sms", "voice"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .json import (
    JsonResource,
    AsyncJsonResource,
    JsonResourceWithRawResponse,
    AsyncJsonResourceWithRawResponse,
    JsonResourceWithStreamingResponse,
    AsyncJsonResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["TranscriptionsResource", "AsyncTranscriptionsResource"]


class TranscriptionsResource(SyncAPIResource):
    @cached_property
    def json(self) -> JsonResource:
        """TeXML REST Commands"""
        return JsonResource(self._client)

    @cached_property
    def with_raw_response(self) -> TranscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TranscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TranscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TranscriptionsResourceWithStreamingResponse(self)


class AsyncTranscriptionsResource(AsyncAPIResource):
    @cached_property
    def json(self) -> AsyncJsonResource:
        """TeXML REST Commands"""
        return AsyncJsonResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTranscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTranscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTranscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTranscriptionsResourceWithStreamingResponse(self)


class TranscriptionsResourceWithRawResponse:
    def __init__(self, transcriptions: TranscriptionsResource) -> None:
        self._transcriptions = transcriptions

    @cached_property
    def json(self) -> JsonResourceWithRawResponse:
        """TeXML REST Commands"""
        return JsonResourceWithRawResponse(self._transcriptions.json)


class AsyncTranscriptionsResourceWithRawResponse:
    def __init__(self, transcriptions: AsyncTranscriptionsResource) -> None:
        self._transcriptions = transcriptions

    @cached_property
    def json(self) -> AsyncJsonResourceWithRawResponse:
        """TeXML REST Commands"""
        return AsyncJsonResourceWithRawResponse(self._transcriptions.json)


class TranscriptionsResourceWithStreamingResponse:
    def __init__(self, transcriptions: TranscriptionsResource) -> None:
        self._transcriptions = transcriptions

    @cached_property
    def json(self) -> JsonResourceWithStreamingResponse:
        """TeXML REST Commands"""
        return JsonResourceWithStreamingResponse(self._transcriptions.json)


class AsyncTranscriptionsResourceWithStreamingResponse:
    def __init__(self, transcriptions: AsyncTranscriptionsResource) -> None:
        self._transcriptions = transcriptions

    @cached_property
    def json(self) -> AsyncJsonResourceWithStreamingResponse:
        """TeXML REST Commands"""
        return AsyncJsonResourceWithStreamingResponse(self._transcriptions.json)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["UserGroupReference"]


class UserGroupReference(BaseModel):
    """A reference to a group that a user belongs to."""

    id: str
    """The unique identifier of the group."""

    name: str
    """The name of the group."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.ai.missions.runs import telnyx_agent_link_params
from .....types.ai.missions.runs.telnyx_agent_link_response import TelnyxAgentLinkResponse
from .....types.ai.missions.runs.telnyx_agent_list_response import TelnyxAgentListResponse

__all__ = ["TelnyxAgentsResource", "AsyncTelnyxAgentsResource"]


class TelnyxAgentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TelnyxAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TelnyxAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TelnyxAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TelnyxAgentsResourceWithStreamingResponse(self)

    def list(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxAgentListResponse:
        """
        List all Telnyx agents linked to a run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}/telnyx-agents",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxAgentListResponse,
        )

    def link(
        self,
        run_id: str,
        *,
        mission_id: str,
        telnyx_agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxAgentLinkResponse:
        """
        Link a Telnyx AI agent (voice/messaging) to a run

        Args:
          telnyx_agent_id: The Telnyx AI agent ID to link

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/telnyx-agents",
            body=maybe_transform({"telnyx_agent_id": telnyx_agent_id}, telnyx_agent_link_params.TelnyxAgentLinkParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxAgentLinkResponse,
        )

    def unlink(
        self,
        telnyx_agent_id: str,
        *,
        mission_id: str,
        run_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Unlink a Telnyx agent from a run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        if not telnyx_agent_id:
            raise ValueError(f"Expected a non-empty value for `telnyx_agent_id` but received {telnyx_agent_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/missions/{mission_id}/runs/{run_id}/telnyx-agents/{telnyx_agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncTelnyxAgentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTelnyxAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTelnyxAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTelnyxAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTelnyxAgentsResourceWithStreamingResponse(self)

    async def list(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxAgentListResponse:
        """
        List all Telnyx agents linked to a run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}/telnyx-agents",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxAgentListResponse,
        )

    async def link(
        self,
        run_id: str,
        *,
        mission_id: str,
        telnyx_agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxAgentLinkResponse:
        """
        Link a Telnyx AI agent (voice/messaging) to a run

        Args:
          telnyx_agent_id: The Telnyx AI agent ID to link

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/telnyx-agents",
            body=await async_maybe_transform(
                {"telnyx_agent_id": telnyx_agent_id}, telnyx_agent_link_params.TelnyxAgentLinkParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxAgentLinkResponse,
        )

    async def unlink(
        self,
        telnyx_agent_id: str,
        *,
        mission_id: str,
        run_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Unlink a Telnyx agent from a run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        if not telnyx_agent_id:
            raise ValueError(f"Expected a non-empty value for `telnyx_agent_id` but received {telnyx_agent_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/missions/{mission_id}/runs/{run_id}/telnyx-agents/{telnyx_agent_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class TelnyxAgentsResourceWithRawResponse:
    def __init__(self, telnyx_agents: TelnyxAgentsResource) -> None:
        self._telnyx_agents = telnyx_agents

        self.list = to_raw_response_wrapper(
            telnyx_agents.list,
        )
        self.link = to_raw_response_wrapper(
            telnyx_agents.link,
        )
        self.unlink = to_raw_response_wrapper(
            telnyx_agents.unlink,
        )


class AsyncTelnyxAgentsResourceWithRawResponse:
    def __init__(self, telnyx_agents: AsyncTelnyxAgentsResource) -> None:
        self._telnyx_agents = telnyx_agents

        self.list = async_to_raw_response_wrapper(
            telnyx_agents.list,
        )
        self.link = async_to_raw_response_wrapper(
            telnyx_agents.link,
        )
        self.unlink = async_to_raw_response_wrapper(
            telnyx_agents.unlink,
        )


class TelnyxAgentsResourceWithStreamingResponse:
    def __init__(self, telnyx_agents: TelnyxAgentsResource) -> None:
        self._telnyx_agents = telnyx_agents

        self.list = to_streamed_response_wrapper(
            telnyx_agents.list,
        )
        self.link = to_streamed_response_wrapper(
            telnyx_agents.link,
        )
        self.unlink = to_streamed_response_wrapper(
            telnyx_agents.unlink,
        )


class AsyncTelnyxAgentsResourceWithStreamingResponse:
    def __init__(self, telnyx_agents: AsyncTelnyxAgentsResource) -> None:
        self._telnyx_agents = telnyx_agents

        self.list = async_to_streamed_response_wrapper(
            telnyx_agents.list,
        )
        self.link = async_to_streamed_response_wrapper(
            telnyx_agents.link,
        )
        self.unlink = async_to_streamed_response_wrapper(
            telnyx_agents.unlink,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["SslCertificate", "IssuedBy", "IssuedTo"]


class IssuedBy(BaseModel):
    common_name: Optional[str] = None
    """The common name of the entity the certificate was issued by"""

    organization: Optional[str] = None
    """The organization the certificate was issued by"""

    organization_unit: Optional[str] = None
    """The organizational unit the certificate was issued by"""


class IssuedTo(BaseModel):
    common_name: Optional[str] = None
    """The common name of the entity the certificate was issued to"""

    organization: Optional[str] = None
    """The organization the certificate was issued to"""

    organization_unit: Optional[str] = None
    """The organizational unit the certificate was issued to"""


class SslCertificate(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the SSL certificate"""

    created_at: Optional[datetime] = None
    """Time when SSL certificate was uploaded"""

    issued_by: Optional[IssuedBy] = None

    issued_to: Optional[IssuedTo] = None

    valid_from: Optional[datetime] = None
    """The time the certificate is valid from"""

    valid_to: Optional[datetime] = None
    """The time the certificate is valid to"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TollFreeVerificationEntityType"]

TollFreeVerificationEntityType: TypeAlias = Literal[
    "SOLE_PROPRIETOR", "PRIVATE_PROFIT", "PUBLIC_PROFIT", "NON_PROFIT", "GOVERNMENT"
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AdvancedOrderCreateResponse"]


class AdvancedOrderCreateResponse(BaseModel):
    id: Optional[str] = None

    area_code: Optional[str] = None

    comments: Optional[str] = None

    country_code: Optional[str] = None

    customer_reference: Optional[str] = None

    features: Optional[List[Literal["sms", "mms", "voice", "fax", "emergency"]]] = None

    orders: Optional[List[str]] = None

    phone_number_type: Optional[
        List[Literal["local", "mobile", "toll_free", "shared_cost", "national", "landline"]]
    ] = None

    quantity: Optional[int] = None

    requirement_group_id: Optional[str] = None
    """The ID of the requirement group associated with this advanced order"""

    status: Optional[List[Literal["pending", "processing", "ordered"]]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .organization_user import OrganizationUser

__all__ = ["UserRetrieveResponse"]


class UserRetrieveResponse(BaseModel):
    data: Optional[OrganizationUser] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_machine_premium_detection_ended import CallMachinePremiumDetectionEnded

__all__ = ["CallMachinePremiumDetectionEndedWebhookEvent"]


class CallMachinePremiumDetectionEndedWebhookEvent(BaseModel):
    data: Optional[CallMachinePremiumDetectionEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel
from .custom_storage_configuration import CustomStorageConfiguration

__all__ = ["CustomStorageCredentialRetrieveResponse"]


class CustomStorageCredentialRetrieveResponse(BaseModel):
    connection_id: str
    """
    Uniquely identifies a Telnyx application (Call Control, TeXML) or Sip connection
    resource.
    """

    data: CustomStorageConfiguration

    record_type: Literal["custom_storage_credentials"]
    """Identifies record type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.porting_orders import (
    VerificationCodeListResponse,
    VerificationCodeVerifyResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVerificationCodes:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        verification_code = client.porting_orders.verification_codes.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(
            SyncDefaultFlatPagination[VerificationCodeListResponse], verification_code, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        verification_code = client.porting_orders.verification_codes.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter={"verified": True},
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(
            SyncDefaultFlatPagination[VerificationCodeListResponse], verification_code, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.porting_orders.verification_codes.with_raw_response.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification_code = response.parse()
        assert_matches_type(
            SyncDefaultFlatPagination[VerificationCodeListResponse], verification_code, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.porting_orders.verification_codes.with_streaming_response.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification_code = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[VerificationCodeListResponse], verification_code, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.verification_codes.with_raw_response.list(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_send(self, client: Telnyx) -> None:
        verification_code = client.porting_orders.verification_codes.send(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert verification_code is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_send_with_all_params(self, client: Telnyx) -> None:
        verification_code = client.porting_orders.verification_codes.send(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            phone_numbers=["+61424000001", "+61424000002"],
            verification_method="sms",
        )
        assert verification_code is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_send(self, client: Telnyx) -> None:
        response = client.porting_orders.verification_codes.with_raw_response.send(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification_code = response.parse()
        assert verification_code is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_send(self, client: Telnyx) -> None:
        with client.porting_orders.verification_codes.with_streaming_response.send(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification_code = response.parse()
            assert verification_code is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_send(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.verification_codes.with_raw_response.send(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_verify(self, client: Telnyx) -> None:
        verification_code = client.porting_orders.verification_codes.verify(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(VerificationCodeVerifyResponse, verification_code, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_verify_with_all_params(self, client: Telnyx) -> None:
        verification_code = client.porting_orders.verification_codes.verify(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            verification_codes=[
                {
                    "code": "12345",
                    "phone_number": "+61424000001",
                },
                {
                    "code": "54321",
                    "phone_number": "+61424000002",
                },
            ],
        )
        assert_matches_type(VerificationCodeVerifyResponse, verification_code, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_verify(self, client: Telnyx) -> None:
        response = client.porting_orders.verification_codes.with_raw_response.verify(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification_code = response.parse()
        assert_matches_type(VerificationCodeVerifyResponse, verification_code, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_verify(self, client: Telnyx) -> None:
        with client.porting_orders.verification_codes.with_streaming_response.verify(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification_code = response.parse()
            assert_matches_type(VerificationCodeVerifyResponse, verification_code, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_verify(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.verification_codes.with_raw_response.verify(
                id="",
            )


class TestAsyncVerificationCodes:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        verification_code = await async_client.porting_orders.verification_codes.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[VerificationCodeListResponse], verification_code, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verification_code = await async_client.porting_orders.verification_codes.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter={"verified": True},
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[VerificationCodeListResponse], verification_code, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.verification_codes.with_raw_response.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification_code = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[VerificationCodeListResponse], verification_code, path=["response"]
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.verification_codes.with_streaming_response.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification_code = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[VerificationCodeListResponse], verification_code, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.verification_codes.with_raw_response.list(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_send(self, async_client: AsyncTelnyx) -> None:
        verification_code = await async_client.porting_orders.verification_codes.send(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert verification_code is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_send_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verification_code = await async_client.porting_orders.verification_codes.send(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            phone_numbers=["+61424000001", "+61424000002"],
            verification_method="sms",
        )
        assert verification_code is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_send(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.verification_codes.with_raw_response.send(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification_code = await response.parse()
        assert verification_code is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_send(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.verification_codes.with_streaming_response.send(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification_code = await response.parse()
            assert verification_code is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_send(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.verification_codes.with_raw_response.send(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_verify(self, async_client: AsyncTelnyx) -> None:
        verification_code = await async_client.porting_orders.verification_codes.verify(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(VerificationCodeVerifyResponse, verification_code, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_verify_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verification_code = await async_client.porting_orders.verification_codes.verify(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            verification_codes=[
                {
                    "code": "12345",
                    "phone_number": "+61424000001",
                },
                {
                    "code": "54321",
                    "phone_number": "+61424000002",
                },
            ],
        )
        assert_matches_type(VerificationCodeVerifyResponse, verification_code, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_verify(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.verification_codes.with_raw_response.verify(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification_code = await response.parse()
        assert_matches_type(VerificationCodeVerifyResponse, verification_code, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_verify(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.verification_codes.with_streaming_response.verify(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification_code = await response.parse()
            assert_matches_type(VerificationCodeVerifyResponse, verification_code, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_verify(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.verification_codes.with_raw_response.verify(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["QueueCreateParams"]


class QueueCreateParams(TypedDict, total=False):
    friendly_name: Annotated[str, PropertyInfo(alias="FriendlyName")]
    """A human readable name for the queue."""

    max_size: Annotated[int, PropertyInfo(alias="MaxSize")]
    """The maximum size of the queue."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.reports import (
    mdr_usage_report_list_params,
    mdr_usage_report_create_params,
    mdr_usage_report_fetch_sync_params,
)
from ...types.reports.mdr_usage_report import MdrUsageReport
from ...types.reports.mdr_usage_report_create_response import MdrUsageReportCreateResponse
from ...types.reports.mdr_usage_report_delete_response import MdrUsageReportDeleteResponse
from ...types.reports.mdr_usage_report_retrieve_response import MdrUsageReportRetrieveResponse
from ...types.reports.mdr_usage_report_fetch_sync_response import MdrUsageReportFetchSyncResponse

__all__ = ["MdrUsageReportsResource", "AsyncMdrUsageReportsResource"]


class MdrUsageReportsResource(SyncAPIResource):
    """Messaging usage reports"""

    @cached_property
    def with_raw_response(self) -> MdrUsageReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MdrUsageReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MdrUsageReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MdrUsageReportsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        aggregation_type: Literal["NO_AGGREGATION", "PROFILE", "TAGS"],
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        profiles: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MdrUsageReportCreateResponse:
        """Submit request for new new messaging usage report.

        This endpoint will pull and
        aggregate messaging data in specified time period.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/reports/mdr_usage_reports",
            body=maybe_transform(
                {
                    "aggregation_type": aggregation_type,
                    "end_date": end_date,
                    "start_date": start_date,
                    "profiles": profiles,
                },
                mdr_usage_report_create_params.MdrUsageReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MdrUsageReportCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MdrUsageReportRetrieveResponse:
        """
        Fetch a single messaging usage report by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/reports/mdr_usage_reports/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MdrUsageReportRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MdrUsageReport]:
        """Fetch all messaging usage reports.

        Usage reports are aggregated messaging data
        for specified time period and breakdown

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/reports/mdr_usage_reports",
            page=SyncDefaultFlatPagination[MdrUsageReport],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    mdr_usage_report_list_params.MdrUsageReportListParams,
                ),
            ),
            model=MdrUsageReport,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MdrUsageReportDeleteResponse:
        """
        Delete messaging usage report by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/reports/mdr_usage_reports/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MdrUsageReportDeleteResponse,
        )

    def fetch_sync(
        self,
        *,
        aggregation_type: Literal["NO_AGGREGATION", "PROFILE", "TAGS"],
        end_date: Union[str, datetime] | Omit = omit,
        profiles: SequenceNotStr[str] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MdrUsageReportFetchSyncResponse:
        """Generate and fetch messaging usage report synchronously.

        This endpoint will both
        generate and fetch the messaging report over a specified time period. No polling
        is necessary but the response may take up to a couple of minutes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/reports/mdr_usage_reports/sync",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "aggregation_type": aggregation_type,
                        "end_date": end_date,
                        "profiles": profiles,
                        "start_date": start_date,
                    },
                    mdr_usage_report_fetch_sync_params.MdrUsageReportFetchSyncParams,
                ),
            ),
            cast_to=MdrUsageReportFetchSyncResponse,
        )


class AsyncMdrUsageReportsResource(AsyncAPIResource):
    """Messaging usage reports"""

    @cached_property
    def with_raw_response(self) -> AsyncMdrUsageReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMdrUsageReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMdrUsageReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMdrUsageReportsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        aggregation_type: Literal["NO_AGGREGATION", "PROFILE", "TAGS"],
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        profiles: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MdrUsageReportCreateResponse:
        """Submit request for new new messaging usage report.

        This endpoint will pull and
        aggregate messaging data in specified time period.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/reports/mdr_usage_reports",
            body=await async_maybe_transform(
                {
                    "aggregation_type": aggregation_type,
                    "end_date": end_date,
                    "start_date": start_date,
                    "profiles": profiles,
                },
                mdr_usage_report_create_params.MdrUsageReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MdrUsageReportCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MdrUsageReportRetrieveResponse:
        """
        Fetch a single messaging usage report by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/reports/mdr_usage_reports/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MdrUsageReportRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MdrUsageReport, AsyncDefaultFlatPagination[MdrUsageReport]]:
        """Fetch all messaging usage reports.

        Usage reports are aggregated messaging data
        for specified time period and breakdown

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/reports/mdr_usage_reports",
            page=AsyncDefaultFlatPagination[MdrUsageReport],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    mdr_usage_report_list_params.MdrUsageReportListParams,
                ),
            ),
            model=MdrUsageReport,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MdrUsageReportDeleteResponse:
        """
        Delete messaging usage report by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/reports/mdr_usage_reports/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MdrUsageReportDeleteResponse,
        )

    async def fetch_sync(
        self,
        *,
        aggregation_type: Literal["NO_AGGREGATION", "PROFILE", "TAGS"],
        end_date: Union[str, datetime] | Omit = omit,
        profiles: SequenceNotStr[str] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MdrUsageReportFetchSyncResponse:
        """Generate and fetch messaging usage report synchronously.

        This endpoint will both
        generate and fetch the messaging report over a specified time period. No polling
        is necessary but the response may take up to a couple of minutes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/reports/mdr_usage_reports/sync",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "aggregation_type": aggregation_type,
                        "end_date": end_date,
                        "profiles": profiles,
                        "start_date": start_date,
                    },
                    mdr_usage_report_fetch_sync_params.MdrUsageReportFetchSyncParams,
                ),
            ),
            cast_to=MdrUsageReportFetchSyncResponse,
        )


class MdrUsageReportsResourceWithRawResponse:
    def __init__(self, mdr_usage_reports: MdrUsageReportsResource) -> None:
        self._mdr_usage_reports = mdr_usage_reports

        self.create = to_raw_response_wrapper(
            mdr_usage_reports.create,
        )
        self.retrieve = to_raw_response_wrapper(
            mdr_usage_reports.retrieve,
        )
        self.list = to_raw_response_wrapper(
            mdr_usage_reports.list,
        )
        self.delete = to_raw_response_wrapper(
            mdr_usage_reports.delete,
        )
        self.fetch_sync = to_raw_response_wrapper(
            mdr_usage_reports.fetch_sync,
        )


class AsyncMdrUsageReportsResourceWithRawResponse:
    def __init__(self, mdr_usage_reports: AsyncMdrUsageReportsResource) -> None:
        self._mdr_usage_reports = mdr_usage_reports

        self.create = async_to_raw_response_wrapper(
            mdr_usage_reports.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            mdr_usage_reports.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            mdr_usage_reports.list,
        )
        self.delete = async_to_raw_response_wrapper(
            mdr_usage_reports.delete,
        )
        self.fetch_sync = async_to_raw_response_wrapper(
            mdr_usage_reports.fetch_sync,
        )


class MdrUsageReportsResourceWithStreamingResponse:
    def __init__(self, mdr_usage_reports: MdrUsageReportsResource) -> None:
        self._mdr_usage_reports = mdr_usage_reports

        self.create = to_streamed_response_wrapper(
            mdr_usage_reports.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            mdr_usage_reports.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            mdr_usage_reports.list,
        )
        self.delete = to_streamed_response_wrapper(
            mdr_usage_reports.delete,
        )
        self.fetch_sync = to_streamed_response_wrapper(
            mdr_usage_reports.fetch_sync,
        )


class AsyncMdrUsageReportsResourceWithStreamingResponse:
    def __init__(self, mdr_usage_reports: AsyncMdrUsageReportsResource) -> None:
        self._mdr_usage_reports = mdr_usage_reports

        self.create = async_to_streamed_response_wrapper(
            mdr_usage_reports.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            mdr_usage_reports.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            mdr_usage_reports.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            mdr_usage_reports.delete,
        )
        self.fetch_sync = async_to_streamed_response_wrapper(
            mdr_usage_reports.fetch_sync,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["SettingUpdateResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Internal ID of Whatsapp business account"""

    name: Optional[str] = None

    record_type: Optional[str] = None

    timezone: Optional[str] = None

    updated_at: Optional[datetime] = None

    webhook_enabled: Optional[bool] = None
    """Enable/disable receiving Whatsapp events"""

    webhook_events: Optional[List[str]] = None

    webhook_failover_url: Optional[str] = None
    """Failover URL to receive Whatsapp events"""

    webhook_url: Optional[str] = None
    """URL to receive Whatsapp events"""


class SettingUpdateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CustomerServiceRecordCreateParams", "AdditionalData"]


class CustomerServiceRecordCreateParams(TypedDict, total=False):
    phone_number: Required[str]
    """A valid US phone number in E164 format."""

    additional_data: AdditionalData

    webhook_url: str
    """Callback URL to receive webhook notifications."""


class AdditionalData(TypedDict, total=False):
    account_number: str
    """The account number of the customer service record."""

    address_line_1: str
    """The first line of the address of the customer service record."""

    authorized_person_name: str
    """The name of the authorized person."""

    billing_phone_number: str
    """The billing phone number of the customer service record."""

    city: str
    """The city of the customer service record."""

    customer_code: str
    """The customer code of the customer service record."""

    name: str
    """The name of the administrator of CSR."""

    pin: str
    """The PIN of the customer service record."""

    state: str
    """The state of the customer service record."""

    zip_code: str
    """The zip code of the customer service record."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ...._models import BaseModel
from .pagination_meta_simple import PaginationMetaSimple

__all__ = ["UsageGetBucketUsageResponse", "Data"]


class Data(BaseModel):
    num_objects: Optional[int] = None
    """The number of objects in the bucket"""

    size: Optional[int] = None
    """The size of the bucket in bytes"""

    size_kb: Optional[int] = None
    """The size of the bucket in kilobytes"""

    timestamp: Optional[datetime] = None
    """The time the snapshot was taken"""


class UsageGetBucketUsageResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMetaSimple] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["PortingOrderPhoneNumberConfiguration"]


class PortingOrderPhoneNumberConfiguration(BaseModel):
    billing_group_id: Optional[str] = None
    """identifies the billing group to set on the numbers when ported"""

    connection_id: Optional[str] = None
    """identifies the connection to set on the numbers when ported"""

    emergency_address_id: Optional[str] = None
    """identifies the emergency address to set on the numbers when ported"""

    messaging_profile_id: Optional[str] = None
    """identifies the messaging profile to set on the numbers when ported"""

    tags: Optional[List[str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .shared.messaging_hosted_number_order import MessagingHostedNumberOrder

__all__ = ["MessagingHostedNumberOrderCreateResponse"]


class MessagingHostedNumberOrderCreateResponse(BaseModel):
    data: Optional[MessagingHostedNumberOrder] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .migrations import (
    MigrationsResource,
    AsyncMigrationsResource,
    MigrationsResourceWithRawResponse,
    AsyncMigrationsResourceWithRawResponse,
    MigrationsResourceWithStreamingResponse,
    AsyncMigrationsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "MigrationsResource",
    "AsyncMigrationsResource",
    "MigrationsResourceWithRawResponse",
    "AsyncMigrationsResourceWithRawResponse",
    "MigrationsResourceWithStreamingResponse",
    "AsyncMigrationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AuthenticationProvider", "Settings"]


class Settings(BaseModel):
    """The settings associated with the authentication provider."""

    assertion_consumer_service_url: Optional[str] = None
    """The Assertion Consumer Service URL for the service provider (Telnyx)."""

    idp_attribute_names: Optional[Dict[str, object]] = None
    """Mapping of SAML attribute names used by the identity provider (IdP)."""

    idp_cert_fingerprint: Optional[str] = None
    """The certificate fingerprint for the identity provider (IdP)"""

    idp_cert_fingerprint_algorithm: Optional[Literal["sha1", "sha256", "sha384", "sha512"]] = None
    """
    The algorithm used to generate the identity provider's (IdP) certificate
    fingerprint
    """

    idp_certificate: Optional[str] = None
    """The full X.509 certificate for the identity provider (IdP)."""

    idp_entity_id: Optional[str] = None
    """The Entity ID for the identity provider (IdP)."""

    idp_slo_target_url: Optional[str] = None
    """The Single Logout (SLO) target URL for the identity provider (IdP)."""

    idp_sso_target_url: Optional[str] = None
    """The SSO target url for the identity provider (IdP)."""

    name_identifier_format: Optional[str] = None
    """The name identifier format associated with the authentication provider.

    This must be the same for both the Identity Provider (IdP) and the service
    provider (Telnyx).
    """

    provision_groups: Optional[bool] = None
    """Whether group provisioning is enabled for this authentication provider."""

    service_provider_entity_id: Optional[str] = None
    """The Entity ID for the service provider (Telnyx)."""

    service_provider_login_url: Optional[str] = None
    """The login URL for the service provider (Telnyx).

    Users navigate to this URL to initiate SSO login.
    """


class AuthenticationProvider(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the authentication provider."""

    activated_at: Optional[datetime] = None
    """
    ISO 8601 formatted date indicating when the authentication provider was
    activated.
    """

    active: Optional[bool] = None
    """The active status of the authentication provider"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    name: Optional[str] = None
    """The name associated with the authentication provider."""

    organization_id: Optional[str] = None
    """The id from the Organization the authentication provider belongs to."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    settings: Optional[Settings] = None
    """The settings associated with the authentication provider."""

    short_name: Optional[str] = None
    """The short name associated with the authentication provider.

    This must be unique and URL-friendly, as it's going to be part of the login URL.
    """

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..managed_account import ManagedAccount

__all__ = ["ActionDisableResponse"]


class ActionDisableResponse(BaseModel):
    data: Optional[ManagedAccount] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    CreateVerificationResponse,
    VerificationRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVerifications:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        verification = client.verifications.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(VerificationRetrieveResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.verifications.with_raw_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification = response.parse()
        assert_matches_type(VerificationRetrieveResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.verifications.with_streaming_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification = response.parse()
            assert_matches_type(VerificationRetrieveResponse, verification, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `verification_id` but received ''"):
            client.verifications.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_trigger_call(self, client: Telnyx) -> None:
        verification = client.verifications.trigger_call(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_trigger_call_with_all_params(self, client: Telnyx) -> None:
        verification = client.verifications.trigger_call(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            custom_code="43612",
            extension="1www2WABCDw9",
            timeout_secs=300,
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_trigger_call(self, client: Telnyx) -> None:
        response = client.verifications.with_raw_response.trigger_call(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification = response.parse()
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_trigger_call(self, client: Telnyx) -> None:
        with client.verifications.with_streaming_response.trigger_call(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification = response.parse()
            assert_matches_type(CreateVerificationResponse, verification, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_trigger_flashcall(self, client: Telnyx) -> None:
        verification = client.verifications.trigger_flashcall(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_trigger_flashcall_with_all_params(self, client: Telnyx) -> None:
        verification = client.verifications.trigger_flashcall(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            timeout_secs=300,
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_trigger_flashcall(self, client: Telnyx) -> None:
        response = client.verifications.with_raw_response.trigger_flashcall(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification = response.parse()
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_trigger_flashcall(self, client: Telnyx) -> None:
        with client.verifications.with_streaming_response.trigger_flashcall(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification = response.parse()
            assert_matches_type(CreateVerificationResponse, verification, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_trigger_sms(self, client: Telnyx) -> None:
        verification = client.verifications.trigger_sms(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_trigger_sms_with_all_params(self, client: Telnyx) -> None:
        verification = client.verifications.trigger_sms(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            custom_code="43612",
            timeout_secs=300,
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_trigger_sms(self, client: Telnyx) -> None:
        response = client.verifications.with_raw_response.trigger_sms(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification = response.parse()
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_trigger_sms(self, client: Telnyx) -> None:
        with client.verifications.with_streaming_response.trigger_sms(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification = response.parse()
            assert_matches_type(CreateVerificationResponse, verification, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncVerifications:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        verification = await async_client.verifications.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(VerificationRetrieveResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verifications.with_raw_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification = await response.parse()
        assert_matches_type(VerificationRetrieveResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verifications.with_streaming_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification = await response.parse()
            assert_matches_type(VerificationRetrieveResponse, verification, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `verification_id` but received ''"):
            await async_client.verifications.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_trigger_call(self, async_client: AsyncTelnyx) -> None:
        verification = await async_client.verifications.trigger_call(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_trigger_call_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verification = await async_client.verifications.trigger_call(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            custom_code="43612",
            extension="1www2WABCDw9",
            timeout_secs=300,
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_trigger_call(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verifications.with_raw_response.trigger_call(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification = await response.parse()
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_trigger_call(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verifications.with_streaming_response.trigger_call(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification = await response.parse()
            assert_matches_type(CreateVerificationResponse, verification, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_trigger_flashcall(self, async_client: AsyncTelnyx) -> None:
        verification = await async_client.verifications.trigger_flashcall(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_trigger_flashcall_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verification = await async_client.verifications.trigger_flashcall(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            timeout_secs=300,
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_trigger_flashcall(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verifications.with_raw_response.trigger_flashcall(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification = await response.parse()
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_trigger_flashcall(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verifications.with_streaming_response.trigger_flashcall(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification = await response.parse()
            assert_matches_type(CreateVerificationResponse, verification, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_trigger_sms(self, async_client: AsyncTelnyx) -> None:
        verification = await async_client.verifications.trigger_sms(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_trigger_sms_with_all_params(self, async_client: AsyncTelnyx) -> None:
        verification = await async_client.verifications.trigger_sms(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
            custom_code="43612",
            timeout_secs=300,
        )
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_trigger_sms(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.verifications.with_raw_response.trigger_sms(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        verification = await response.parse()
        assert_matches_type(CreateVerificationResponse, verification, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_trigger_sms(self, async_client: AsyncTelnyx) -> None:
        async with async_client.verifications.with_streaming_response.trigger_sms(
            phone_number="+13035551234",
            verify_profile_id="12ade33a-21c0-473b-b055-b3c836e1c292",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            verification = await response.parse()
            assert_matches_type(CreateVerificationResponse, verification, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .export_porting_orders_csv_report import ExportPortingOrdersCsvReport

__all__ = ["PortingReport"]


class PortingReport(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the report."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    document_id: Optional[str] = None
    """Identifies the document that was uploaded when report was generated.

    This field is only populated when the report is under completed status.
    """

    params: Optional[ExportPortingOrdersCsvReport] = None
    """The parameters for generating a porting orders CSV report."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    report_type: Optional[Literal["export_porting_orders_csv"]] = None
    """Identifies the type of report"""

    status: Optional[Literal["pending", "completed"]] = None
    """The current status of the report generation."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .purchase_create_params import PurchaseCreateParams as PurchaseCreateParams
from .register_create_params import RegisterCreateParams as RegisterCreateParams
from .purchase_create_response import PurchaseCreateResponse as PurchaseCreateResponse
from .register_create_response import RegisterCreateResponse as RegisterCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.storage.buckets import usage_get_api_usage_params
from ....types.storage.buckets.usage_get_api_usage_response import UsageGetAPIUsageResponse
from ....types.storage.buckets.usage_get_bucket_usage_response import UsageGetBucketUsageResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    """Bucket Usage operations"""

    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def get_api_usage(
        self,
        bucket_name: str,
        *,
        filter: usage_get_api_usage_params.Filter,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageGetAPIUsageResponse:
        """
        Returns the detail on API usage on a bucket of a particular time period, group
        by method category.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[start_time], filter[end_time]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._get(
            f"/storage/buckets/{bucket_name}/usage/api",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, usage_get_api_usage_params.UsageGetAPIUsageParams),
            ),
            cast_to=UsageGetAPIUsageResponse,
        )

    def get_bucket_usage(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageGetBucketUsageResponse:
        """
        Returns the amount of storage space and number of files a bucket takes up.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._get(
            f"/storage/buckets/{bucket_name}/usage/storage",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageGetBucketUsageResponse,
        )


class AsyncUsageResource(AsyncAPIResource):
    """Bucket Usage operations"""

    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    async def get_api_usage(
        self,
        bucket_name: str,
        *,
        filter: usage_get_api_usage_params.Filter,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageGetAPIUsageResponse:
        """
        Returns the detail on API usage on a bucket of a particular time period, group
        by method category.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[start_time], filter[end_time]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._get(
            f"/storage/buckets/{bucket_name}/usage/api",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter}, usage_get_api_usage_params.UsageGetAPIUsageParams
                ),
            ),
            cast_to=UsageGetAPIUsageResponse,
        )

    async def get_bucket_usage(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageGetBucketUsageResponse:
        """
        Returns the amount of storage space and number of files a bucket takes up.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._get(
            f"/storage/buckets/{bucket_name}/usage/storage",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageGetBucketUsageResponse,
        )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.get_api_usage = to_raw_response_wrapper(
            usage.get_api_usage,
        )
        self.get_bucket_usage = to_raw_response_wrapper(
            usage.get_bucket_usage,
        )


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.get_api_usage = async_to_raw_response_wrapper(
            usage.get_api_usage,
        )
        self.get_bucket_usage = async_to_raw_response_wrapper(
            usage.get_bucket_usage,
        )


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.get_api_usage = to_streamed_response_wrapper(
            usage.get_api_usage,
        )
        self.get_bucket_usage = to_streamed_response_wrapper(
            usage.get_bucket_usage,
        )


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.get_api_usage = async_to_streamed_response_wrapper(
            usage.get_api_usage,
        )
        self.get_bucket_usage = async_to_streamed_response_wrapper(
            usage.get_bucket_usage,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["SiprecSiprecSidJsonResponse"]


class SiprecSiprecSidJsonResponse(BaseModel):
    account_sid: Optional[str] = None
    """The id of the account the resource belongs to."""

    call_sid: Optional[str] = None
    """The id of the call the resource belongs to."""

    date_updated: Optional[str] = None
    """The date and time the siprec session was last updated."""

    error_code: Optional[str] = None
    """The error code of the siprec session."""

    sid: Optional[str] = None
    """The SID of the siprec session."""

    status: Optional[Literal["in-progress", "stopped"]] = None
    """The status of the siprec session."""

    uri: Optional[str] = None
    """The URI of the siprec session."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .runs import (
    RunsResource,
    AsyncRunsResource,
    RunsResourceWithRawResponse,
    AsyncRunsResourceWithRawResponse,
    RunsResourceWithStreamingResponse,
    AsyncRunsResourceWithStreamingResponse,
)
from ......_types import Body, Query, Headers, NotGiven, not_given
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_base_client import make_request_options
from ......types.ai.assistants.tests.test_suite_list_response import TestSuiteListResponse

__all__ = ["TestSuitesResource", "AsyncTestSuitesResource"]


class TestSuitesResource(SyncAPIResource):
    __test__ = False
    """Configure AI assistant specifications"""

    @cached_property
    def runs(self) -> RunsResource:
        """Configure AI assistant specifications"""
        return RunsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TestSuitesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TestSuitesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TestSuitesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TestSuitesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestSuiteListResponse:
        """Retrieves a list of all distinct test suite names available to the current user"""
        return self._get(
            "/ai/assistants/tests/test-suites",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestSuiteListResponse,
        )


class AsyncTestSuitesResource(AsyncAPIResource):
    """Configure AI assistant specifications"""

    @cached_property
    def runs(self) -> AsyncRunsResource:
        """Configure AI assistant specifications"""
        return AsyncRunsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTestSuitesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTestSuitesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTestSuitesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTestSuitesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestSuiteListResponse:
        """Retrieves a list of all distinct test suite names available to the current user"""
        return await self._get(
            "/ai/assistants/tests/test-suites",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestSuiteListResponse,
        )


class TestSuitesResourceWithRawResponse:
    __test__ = False

    def __init__(self, test_suites: TestSuitesResource) -> None:
        self._test_suites = test_suites

        self.list = to_raw_response_wrapper(
            test_suites.list,
        )

    @cached_property
    def runs(self) -> RunsResourceWithRawResponse:
        """Configure AI assistant specifications"""
        return RunsResourceWithRawResponse(self._test_suites.runs)


class AsyncTestSuitesResourceWithRawResponse:
    def __init__(self, test_suites: AsyncTestSuitesResource) -> None:
        self._test_suites = test_suites

        self.list = async_to_raw_response_wrapper(
            test_suites.list,
        )

    @cached_property
    def runs(self) -> AsyncRunsResourceWithRawResponse:
        """Configure AI assistant specifications"""
        return AsyncRunsResourceWithRawResponse(self._test_suites.runs)


class TestSuitesResourceWithStreamingResponse:
    __test__ = False

    def __init__(self, test_suites: TestSuitesResource) -> None:
        self._test_suites = test_suites

        self.list = to_streamed_response_wrapper(
            test_suites.list,
        )

    @cached_property
    def runs(self) -> RunsResourceWithStreamingResponse:
        """Configure AI assistant specifications"""
        return RunsResourceWithStreamingResponse(self._test_suites.runs)


class AsyncTestSuitesResourceWithStreamingResponse:
    def __init__(self, test_suites: AsyncTestSuitesResource) -> None:
        self._test_suites = test_suites

        self.list = async_to_streamed_response_wrapper(
            test_suites.list,
        )

    @cached_property
    def runs(self) -> AsyncRunsResourceWithStreamingResponse:
        """Configure AI assistant specifications"""
        return AsyncRunsResourceWithStreamingResponse(self._test_suites.runs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import access_ip_range_list_params, access_ip_range_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.access_ip_range import AccessIPRange

__all__ = ["AccessIPRangesResource", "AsyncAccessIPRangesResource"]


class AccessIPRangesResource(SyncAPIResource):
    """IP Range Operations"""

    @cached_property
    def with_raw_response(self) -> AccessIPRangesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AccessIPRangesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccessIPRangesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AccessIPRangesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        cidr_block: str,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessIPRange:
        """
        Create new Access IP Range

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/access_ip_ranges",
            body=maybe_transform(
                {
                    "cidr_block": cidr_block,
                    "description": description,
                },
                access_ip_range_create_params.AccessIPRangeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccessIPRange,
        )

    def list(
        self,
        *,
        filter: access_ip_range_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[AccessIPRange]:
        """
        List all Access IP Ranges

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[cidr_block], filter[cidr_block][startswith],
              filter[cidr_block][endswith], filter[cidr_block][contains], filter[created_at].
              Supports complex bracket operations for dynamic filtering.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/access_ip_ranges",
            page=SyncDefaultFlatPagination[AccessIPRange],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    access_ip_range_list_params.AccessIPRangeListParams,
                ),
            ),
            model=AccessIPRange,
        )

    def delete(
        self,
        access_ip_range_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessIPRange:
        """
        Delete access IP ranges

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not access_ip_range_id:
            raise ValueError(f"Expected a non-empty value for `access_ip_range_id` but received {access_ip_range_id!r}")
        return self._delete(
            f"/access_ip_ranges/{access_ip_range_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccessIPRange,
        )


class AsyncAccessIPRangesResource(AsyncAPIResource):
    """IP Range Operations"""

    @cached_property
    def with_raw_response(self) -> AsyncAccessIPRangesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccessIPRangesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccessIPRangesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAccessIPRangesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        cidr_block: str,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessIPRange:
        """
        Create new Access IP Range

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/access_ip_ranges",
            body=await async_maybe_transform(
                {
                    "cidr_block": cidr_block,
                    "description": description,
                },
                access_ip_range_create_params.AccessIPRangeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccessIPRange,
        )

    def list(
        self,
        *,
        filter: access_ip_range_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AccessIPRange, AsyncDefaultFlatPagination[AccessIPRange]]:
        """
        List all Access IP Ranges

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[cidr_block], filter[cidr_block][startswith],
              filter[cidr_block][endswith], filter[cidr_block][contains], filter[created_at].
              Supports complex bracket operations for dynamic filtering.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/access_ip_ranges",
            page=AsyncDefaultFlatPagination[AccessIPRange],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    access_ip_range_list_params.AccessIPRangeListParams,
                ),
            ),
            model=AccessIPRange,
        )

    async def delete(
        self,
        access_ip_range_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessIPRange:
        """
        Delete access IP ranges

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not access_ip_range_id:
            raise ValueError(f"Expected a non-empty value for `access_ip_range_id` but received {access_ip_range_id!r}")
        return await self._delete(
            f"/access_ip_ranges/{access_ip_range_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccessIPRange,
        )


class AccessIPRangesResourceWithRawResponse:
    def __init__(self, access_ip_ranges: AccessIPRangesResource) -> None:
        self._access_ip_ranges = access_ip_ranges

        self.create = to_raw_response_wrapper(
            access_ip_ranges.create,
        )
        self.list = to_raw_response_wrapper(
            access_ip_ranges.list,
        )
        self.delete = to_raw_response_wrapper(
            access_ip_ranges.delete,
        )


class AsyncAccessIPRangesResourceWithRawResponse:
    def __init__(self, access_ip_ranges: AsyncAccessIPRangesResource) -> None:
        self._access_ip_ranges = access_ip_ranges

        self.create = async_to_raw_response_wrapper(
            access_ip_ranges.create,
        )
        self.list = async_to_raw_response_wrapper(
            access_ip_ranges.list,
        )
        self.delete = async_to_raw_response_wrapper(
            access_ip_ranges.delete,
        )


class AccessIPRangesResourceWithStreamingResponse:
    def __init__(self, access_ip_ranges: AccessIPRangesResource) -> None:
        self._access_ip_ranges = access_ip_ranges

        self.create = to_streamed_response_wrapper(
            access_ip_ranges.create,
        )
        self.list = to_streamed_response_wrapper(
            access_ip_ranges.list,
        )
        self.delete = to_streamed_response_wrapper(
            access_ip_ranges.delete,
        )


class AsyncAccessIPRangesResourceWithStreamingResponse:
    def __init__(self, access_ip_ranges: AsyncAccessIPRangesResource) -> None:
        self._access_ip_ranges = access_ip_ranges

        self.create = async_to_streamed_response_wrapper(
            access_ip_ranges.create,
        )
        self.list = async_to_streamed_response_wrapper(
            access_ip_ranges.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            access_ip_ranges.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PaymentCreateStoredPaymentTransactionResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None

    amount_cents: Optional[int] = None

    amount_currency: Optional[str] = None

    auto_recharge: Optional[bool] = None

    created_at: Optional[datetime] = None

    processor_status: Optional[str] = None

    record_type: Optional[Literal["transaction"]] = None

    transaction_processing_type: Optional[Literal["stored_payment"]] = None


class PaymentCreateStoredPaymentTransactionResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MinimaxVoiceSettings"]


class MinimaxVoiceSettings(BaseModel):
    type: Literal["minimax"]
    """Voice settings provider type"""

    language_boost: Optional[
        Literal[
            "auto",
            "Chinese",
            "Chinese,Yue",
            "English",
            "Arabic",
            "Russian",
            "Spanish",
            "French",
            "Portuguese",
            "German",
            "Turkish",
            "Dutch",
            "Ukrainian",
            "Vietnamese",
            "Indonesian",
            "Japanese",
            "Italian",
            "Korean",
            "Thai",
            "Polish",
            "Romanian",
            "Greek",
            "Czech",
            "Finnish",
            "Hindi",
            "Bulgarian",
            "Danish",
            "Hebrew",
            "Malay",
            "Persian",
            "Slovak",
            "Swedish",
            "Croatian",
            "Filipino",
            "Hungarian",
            "Norwegian",
            "Slovenian",
            "Catalan",
            "Nynorsk",
            "Tamil",
            "Afrikaans",
        ]
    ] = None
    """
    Enhances recognition for specific languages and dialects during MiniMax TTS
    synthesis. Default is null (no boost). Set to 'auto' for automatic language
    detection.
    """

    pitch: Optional[int] = None
    """Voice pitch adjustment. Default is 0."""

    speed: Optional[float] = None
    """Speech speed multiplier. Default is 1.0."""

    vol: Optional[float] = None
    """Speech volume multiplier. Default is 1.0."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .photo import (
    PhotoResource,
    AsyncPhotoResource,
    PhotoResourceWithRawResponse,
    AsyncPhotoResourceWithRawResponse,
    PhotoResourceWithStreamingResponse,
    AsyncPhotoResourceWithStreamingResponse,
)
from .profile import (
    ProfileResource,
    AsyncProfileResource,
    ProfileResourceWithRawResponse,
    AsyncProfileResourceWithRawResponse,
    ProfileResourceWithStreamingResponse,
    AsyncProfileResourceWithStreamingResponse,
)

__all__ = [
    "PhotoResource",
    "AsyncPhotoResource",
    "PhotoResourceWithRawResponse",
    "AsyncPhotoResourceWithRawResponse",
    "PhotoResourceWithStreamingResponse",
    "AsyncPhotoResourceWithStreamingResponse",
    "ProfileResource",
    "AsyncProfileResource",
    "ProfileResourceWithRawResponse",
    "AsyncProfileResourceWithRawResponse",
    "ProfileResourceWithStreamingResponse",
    "AsyncProfileResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.assistants import tag_add_params
from ....types.ai.assistants.tag_add_response import TagAddResponse
from ....types.ai.assistants.tag_list_response import TagListResponse
from ....types.ai.assistants.tag_remove_response import TagRemoveResponse

__all__ = ["TagsResource", "AsyncTagsResource"]


class TagsResource(SyncAPIResource):
    """Configure AI assistant specifications"""

    @cached_property
    def with_raw_response(self) -> TagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TagsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagListResponse:
        """Get All Tags"""
        return self._get(
            "/ai/assistants/tags",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagListResponse,
        )

    def add(
        self,
        assistant_id: str,
        *,
        tag: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagAddResponse:
        """
        Add Assistant Tag

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return self._post(
            f"/ai/assistants/{assistant_id}/tags",
            body=maybe_transform({"tag": tag}, tag_add_params.TagAddParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagAddResponse,
        )

    def remove(
        self,
        tag: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagRemoveResponse:
        """
        Remove Assistant Tag

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not tag:
            raise ValueError(f"Expected a non-empty value for `tag` but received {tag!r}")
        return self._delete(
            f"/ai/assistants/{assistant_id}/tags/{tag}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagRemoveResponse,
        )


class AsyncTagsResource(AsyncAPIResource):
    """Configure AI assistant specifications"""

    @cached_property
    def with_raw_response(self) -> AsyncTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTagsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagListResponse:
        """Get All Tags"""
        return await self._get(
            "/ai/assistants/tags",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagListResponse,
        )

    async def add(
        self,
        assistant_id: str,
        *,
        tag: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagAddResponse:
        """
        Add Assistant Tag

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return await self._post(
            f"/ai/assistants/{assistant_id}/tags",
            body=await async_maybe_transform({"tag": tag}, tag_add_params.TagAddParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagAddResponse,
        )

    async def remove(
        self,
        tag: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TagRemoveResponse:
        """
        Remove Assistant Tag

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not tag:
            raise ValueError(f"Expected a non-empty value for `tag` but received {tag!r}")
        return await self._delete(
            f"/ai/assistants/{assistant_id}/tags/{tag}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TagRemoveResponse,
        )


class TagsResourceWithRawResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.list = to_raw_response_wrapper(
            tags.list,
        )
        self.add = to_raw_response_wrapper(
            tags.add,
        )
        self.remove = to_raw_response_wrapper(
            tags.remove,
        )


class AsyncTagsResourceWithRawResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.list = async_to_raw_response_wrapper(
            tags.list,
        )
        self.add = async_to_raw_response_wrapper(
            tags.add,
        )
        self.remove = async_to_raw_response_wrapper(
            tags.remove,
        )


class TagsResourceWithStreamingResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.list = to_streamed_response_wrapper(
            tags.list,
        )
        self.add = to_streamed_response_wrapper(
            tags.add,
        )
        self.remove = to_streamed_response_wrapper(
            tags.remove,
        )


class AsyncTagsResourceWithStreamingResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.list = async_to_streamed_response_wrapper(
            tags.list,
        )
        self.add = async_to_streamed_response_wrapper(
            tags.add,
        )
        self.remove = async_to_streamed_response_wrapper(
            tags.remove,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .vertical import Vertical as Vertical
from .entity_type import EntityType as EntityType
from .task_status import TaskStatus as TaskStatus
from .telnyx_brand import TelnyxBrand as TelnyxBrand
from .stock_exchange import StockExchange as StockExchange
from .brand_list_params import BrandListParams as BrandListParams
from .brand_create_params import BrandCreateParams as BrandCreateParams
from .brand_list_response import BrandListResponse as BrandListResponse
from .brand_update_params import BrandUpdateParams as BrandUpdateParams
from .telnyx_campaign_csp import TelnyxCampaignCsp as TelnyxCampaignCsp
from .alt_business_id_type import AltBusinessIDType as AltBusinessIDType
from .campaign_list_params import CampaignListParams as CampaignListParams
from .brand_identity_status import BrandIdentityStatus as BrandIdentityStatus
from .phone_number_campaign import PhoneNumberCampaign as PhoneNumberCampaign
from .campaign_list_response import CampaignListResponse as CampaignListResponse
from .campaign_update_params import CampaignUpdateParams as CampaignUpdateParams
from .brand_retrieve_response import BrandRetrieveResponse as BrandRetrieveResponse
from .campaign_sharing_status import CampaignSharingStatus as CampaignSharingStatus
from .brand_optional_attributes import BrandOptionalAttributes as BrandOptionalAttributes
from .telnyx_downstream_campaign import TelnyxDownstreamCampaign as TelnyxDownstreamCampaign
from .brand_get_feedback_response import BrandGetFeedbackResponse as BrandGetFeedbackResponse
from .brand_verify_sms_otp_params import BrandVerifySMSOtpParams as BrandVerifySMSOtpParams
from .brand_trigger_sms_otp_params import BrandTriggerSMSOtpParams as BrandTriggerSMSOtpParams
from .campaign_deactivate_response import CampaignDeactivateResponse as CampaignDeactivateResponse
from .partner_campaign_list_params import PartnerCampaignListParams as PartnerCampaignListParams
from .campaign_submit_appeal_params import CampaignSubmitAppealParams as CampaignSubmitAppealParams
from .brand_trigger_sms_otp_response import BrandTriggerSMSOtpResponse as BrandTriggerSMSOtpResponse
from .campaign_builder_submit_params import CampaignBuilderSubmitParams as CampaignBuilderSubmitParams
from .partner_campaign_update_params import PartnerCampaignUpdateParams as PartnerCampaignUpdateParams
from .campaign_submit_appeal_response import CampaignSubmitAppealResponse as CampaignSubmitAppealResponse
from .campaign_accept_sharing_response import CampaignAcceptSharingResponse as CampaignAcceptSharingResponse
from .profile_assignment_phone_numbers import ProfileAssignmentPhoneNumbers as ProfileAssignmentPhoneNumbers
from .phone_number_campaign_list_params import PhoneNumberCampaignListParams as PhoneNumberCampaignListParams
from .campaign_get_mno_metadata_response import CampaignGetMnoMetadataResponse as CampaignGetMnoMetadataResponse
from .phone_number_campaign_create_params import PhoneNumberCampaignCreateParams as PhoneNumberCampaignCreateParams
from .phone_number_campaign_update_params import PhoneNumberCampaignUpdateParams as PhoneNumberCampaignUpdateParams
from .campaign_get_sharing_status_response import CampaignGetSharingStatusResponse as CampaignGetSharingStatusResponse
from .brand_get_sms_otp_by_reference_params import BrandGetSMSOtpByReferenceParams as BrandGetSMSOtpByReferenceParams
from .brand_retrieve_sms_otp_status_response import (
    BrandRetrieveSMSOtpStatusResponse as BrandRetrieveSMSOtpStatusResponse,
)
from .campaign_get_operation_status_response import (
    CampaignGetOperationStatusResponse as CampaignGetOperationStatusResponse,
)
from .brand_get_sms_otp_by_reference_response import (
    BrandGetSMSOtpByReferenceResponse as BrandGetSMSOtpByReferenceResponse,
)
from .partner_campaign_list_shared_by_me_params import (
    PartnerCampaignListSharedByMeParams as PartnerCampaignListSharedByMeParams,
)
from .partner_campaign_list_shared_by_me_response import (
    PartnerCampaignListSharedByMeResponse as PartnerCampaignListSharedByMeResponse,
)
from .phone_number_assignment_by_profile_assign_params import (
    PhoneNumberAssignmentByProfileAssignParams as PhoneNumberAssignmentByProfileAssignParams,
)
from .partner_campaign_retrieve_sharing_status_response import (
    PartnerCampaignRetrieveSharingStatusResponse as PartnerCampaignRetrieveSharingStatusResponse,
)
from .phone_number_assignment_by_profile_assign_response import (
    PhoneNumberAssignmentByProfileAssignResponse as PhoneNumberAssignmentByProfileAssignResponse,
)
from .phone_number_assignment_by_profile_retrieve_status_response import (
    PhoneNumberAssignmentByProfileRetrieveStatusResponse as PhoneNumberAssignmentByProfileRetrieveStatusResponse,
)
from .phone_number_assignment_by_profile_list_phone_number_status_params import (
    PhoneNumberAssignmentByProfileListPhoneNumberStatusParams as PhoneNumberAssignmentByProfileListPhoneNumberStatusParams,
)
from .phone_number_assignment_by_profile_list_phone_number_status_response import (
    PhoneNumberAssignmentByProfileListPhoneNumberStatusResponse as PhoneNumberAssignmentByProfileListPhoneNumberStatusResponse,
)
from .phone_number_assignment_by_profile_retrieve_phone_number_status_params import (
    PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusParams as PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusParams,
)
from .phone_number_assignment_by_profile_retrieve_phone_number_status_response import (
    PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse as PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["WhatsappReaction"]


class WhatsappReaction(BaseModel):
    emoji: Optional[str] = None

    message_id: Optional[str] = None
